@@ -1,6 +1,6 @@
 /*  
- *  ScreenShot plug-in v0.4 by Sven Neumann, neumanns@uni-duesseldorf.de  
- *  1998/03/13
+ *  ScreenShot plug-in v0.5 by Sven Neumann, neumanns@uni-duesseldorf.de  
+ *  1998/04/02
  *
  *  Any suggestions, bug-reports or patches are very welcome.
  * 
@@ -33,13 +33,10 @@
  *  (98/02/19)  v0.2   small bugfix 
  *  (98/03/09)  v0.3   another one
  *  (98/03/13)  v0.4   cosmetic changes to the dialog
+ *  (98/04/02)  v0.5   it works non-interactively now and registers
+ *                     itself correctly as extension
  */
 
-/* TODO List
- * - make it work non-interactively 
- *   (How? Probably by handing over the window-id? Is this really useful??)
- */  
-
 #include <stdio.h>
 #include <errno.h>
 #include <sys/wait.h>
@@ -48,22 +45,22 @@
 #include "libgimp/gimp.h"
 
 /* Defines */
-#define PLUG_IN_NAME        "plug_in_screenshot"
+#define PLUG_IN_NAME        "extension_screenshot"
 #define PLUG_IN_PRINT_NAME  "Screen Shot"
-#define PLUG_IN_VERSION     "v0.4 (98/03/13)"
+#define PLUG_IN_VERSION     "v0.5 (98/04/02)"
 #define PLUG_IN_MENU_PATH   "<Toolbox>/Xtns/Screen Shot"
 #define PLUG_IN_AUTHOR      "Sven Neumann (neumanns@uni-duesseldorf.de)"
 #define PLUG_IN_COPYRIGHT   "Sven Neumann"
 #define PLUG_IN_DESCRIBTION "Create a screenshot of a single window or the whole screen"
-#define PLUG_IN_HELP        "This plug-in serves as a simple frontend to the X-window utility xwd and the xwd-file-plug-in. After specifying several options, xwd is called, the user selects a window, and the resulting image is loaded into the gimp. It does not work non-interactively."
+#define PLUG_IN_HELP        "This extension serves as a simple frontend to the X-window utility xwd and the xwd-file-plug-in. After specifying several options, xwd is called, the user selects a window, and the resulting image is loaded into the gimp. When called non-interactively it may grab the root window or use the window-id passed as a parameter."
 
 #define NUMBER_IN_ARGS 3
-#define IN_ARGS { PARAM_INT32,    "run_mode", "Interactive, non-interactive" },\
-                { PARAM_INT32,    "root", "Root window { TRUE, FALSE }" },\
-                { PARAM_INT32,    "decor", "Decorations { TRUE, FALSE }" }
+#define IN_ARGS { PARAM_INT32,    "run_mode",  "Interactive, non-interactive" },\
+                { PARAM_INT32,    "root",      "Root window { TRUE, FALSE }" },\
+                { PARAM_STRING,   "window_id", "Window id" }
 
 #define NUMBER_OUT_ARGS 1 
-#define OUT_ARGS { PARAM_IMAGE,   "image", "Output image" }
+#define OUT_ARGS { PARAM_IMAGE,   "image",     "Output image" }
 
 #ifndef XWD
 #define XWD "xwd"
@@ -71,6 +68,7 @@
 
 typedef struct {
   gint root;
+  gchar *window_id;
   gint decor;
 } ScreenShotValues;
 
@@ -81,6 +79,7 @@ typedef struct {
 static ScreenShotValues shootvals = 
 { 
   FALSE,    /* default to window */
+  NULL,      
   TRUE      /* default to decorations */
 };
 
@@ -94,14 +93,14 @@ static void  query (void);
 static void  run (gchar *name,
 		  gint nparams,	          /* number of parameters passed in */
 		  GParam * param,	  /* parameters passed in */
-		  gint *nreturn_vals,      /* number of parameters returned */
+		  gint *nreturn_vals,     /* number of parameters returned */
 		  GParam ** return_vals); /* parameters to be returned */
 static void  shoot (void);
 static gint  shoot_dialog (void);
 static void  shoot_close_callback (GtkWidget *widget,
 				   gpointer   data);
 static void  shoot_ok_callback (GtkWidget *widget,
-			       gpointer   data);
+				gpointer   data);
 static void  shoot_toggle_update (GtkWidget *widget,
 				  gpointer   radio_button);
 static void  shoot_display_image (gint32 image);
@@ -124,26 +123,25 @@ MAIN ()
 
 static void query (void)
 {
+  static GParamDef args[] = { IN_ARGS };
+  static gint nargs = NUMBER_IN_ARGS;
+  static GParamDef return_vals[] = { OUT_ARGS };
+  static gint nreturn_vals = NUMBER_OUT_ARGS;
 
-static GParamDef args[] = { IN_ARGS };
-static gint nargs = NUMBER_IN_ARGS;
-static GParamDef return_vals[] = { OUT_ARGS };
-static gint nreturn_vals = NUMBER_OUT_ARGS;
-
-/* the actual installation of the plugin */
-gimp_install_procedure (PLUG_IN_NAME,
-			PLUG_IN_DESCRIBTION,
-			PLUG_IN_HELP,
-			PLUG_IN_AUTHOR,
-			PLUG_IN_COPYRIGHT,
-			PLUG_IN_VERSION,
-			PLUG_IN_MENU_PATH,
-			NULL,
-			PROC_EXTENSION,		
-			nargs,
-			nreturn_vals,
-			args,
-			return_vals);
+  /* the actual installation of the plugin */
+  gimp_install_procedure (PLUG_IN_NAME,
+			  PLUG_IN_DESCRIBTION,
+			  PLUG_IN_HELP,
+			  PLUG_IN_AUTHOR,
+			  PLUG_IN_COPYRIGHT,
+			  PLUG_IN_VERSION,
+			  PLUG_IN_MENU_PATH,
+			  NULL,
+			  PROC_EXTENSION,		
+			  nargs,
+			  nreturn_vals,
+			  args,
+			  return_vals);
 }
 
 static void 
@@ -176,6 +174,7 @@ run (gchar *name,		/* name of plugin */
     case RUN_INTERACTIVE:
       /* Possibly retrieve data from a previous run */
       gimp_get_data (PLUG_IN_NAME, &shootvals);
+      shootvals.window_id = NULL;
 
       /* Get information from the dialog */
       if (!shoot_dialog())
@@ -183,10 +182,16 @@ run (gchar *name,		/* name of plugin */
       break;
 
     case RUN_NONINTERACTIVE:
-      /* right now, I don't know how to handle non-interactive calls */
-      status = STATUS_CALLING_ERROR;
+      if (nparams == NUMBER_IN_ARGS)
+	{
+	  shootvals.root      = (gint) param[1].data.d_int32;
+	  shootvals.window_id = (gchar*) param[2].data.d_string;
+	  shootvals.decor     = FALSE;
+	}
+      else
+	status = STATUS_CALLING_ERROR;
       break;
-      
+     
     case RUN_WITH_LAST_VALS:
       /* Possibly retrieve data from a previous run */
       gimp_get_data (PLUG_IN_NAME, &shootvals);
@@ -231,7 +236,7 @@ shoot (void)
   GParam *params;
   gint retvals;
   char *tmpname;
-  char *xwdargv[5]; /* only need a maximum of 5 arguments to xwd */
+  char *xwdargv[7]; /* only need a maximum of 7 arguments to xwd */
   gint pid;
   gint status;
   gint i = 0;
@@ -248,12 +253,16 @@ shoot (void)
   xwdargv[i++] = "-out";
   xwdargv[i++] = tmpname;
   if ( shootvals.root == TRUE )
+    xwdargv[i++] = "-root";
+  else 
     {
-      xwdargv[i++] = "-root";
-    }
-  else if (shootvals.decor == TRUE )
-    {
-      xwdargv[i++] = "-frame";
+      if (shootvals.decor == TRUE )
+	xwdargv[i++] = "-frame";
+      if (shootvals.window_id != NULL)
+	{
+	  xwdargv[i++] = "-id";
+	  xwdargv[i++] = shootvals.window_id;
+	}
     }
   xwdargv[i] = NULL;
   
