@@ -1,9 +1,9 @@
 /*
- * "$Id: sharpen.c,v 1.1 1998/02/20 10:41:24 yosh Exp $"
+ * "$Id: sharpen.c,v 1.2 1998/03/13 14:31:20 neo Exp $"
  *
  *   Sharpen filters for The GIMP -- an image manipulation program
  *
- *   Copyright 1997 Michael Sweet.
+ *   Copyright 1997-1998 Michael Sweet (mike@easysw.com)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -40,31 +40,14 @@
  * Revision History:
  *
  *   $Log: sharpen.c,v $
- *   Revision 1.1  1998/02/20 10:41:24  yosh
- *   Initial stable/unstable split done.
+ *   Revision 1.2  1998/03/13 14:31:20  neo
+ *   Updated screenshot and sharpen with versions from the registry
  *
- *   -Yosh
+ *   --Sven
  *
- *   Revision 1.2  1998/01/25 02:18:43  yosh
- *   Applied Sven's menu patch
- *
- *   -Yosh
- *
- *   Revision 1.1.1.1  1997/11/24 22:04:37  sopwith
- *   Let's try this import one last time.
- *
- *   Revision 1.3  1997/11/18 03:04:29  nobody
- *   fixed ugly comment-bugs introduced by evil darkwing
- *   keep out configuration empty dirs
- *   	--darkwing
- *
- *   Revision 1.2  1997/11/17 05:44:06  nobody
- *   updated ChangeLog
- *   dropped non-working doc/Makefile entries
- *   applied many fixes from the registry as well as the devel ML
- *   applied missing patches by Art Haas
- *
- *   	--darkwing
+ *   Revision 1.7  1998/03/12  18:48:52  mike
+ *   Fixed pixel errors around the edge of the bounding rectangle - the
+ *   original pixels weren't being written back to the image...
  *
  *   Revision 1.6  1997/11/14  17:17:59  mike
  *   Updated to dynamically allocate return params in the run() function.
@@ -104,7 +87,7 @@
  */
 
 #define PLUG_IN_NAME		"plug_in_sharpen"
-#define PLUG_IN_VERSION		"1.0.4 - 14 November 1997"
+#define PLUG_IN_VERSION		"1.1 - 11 March 1998"
 #define PREVIEW_SIZE		128
 #define SCALE_WIDTH		64
 #define ENTRY_WIDTH		64
@@ -455,6 +438,17 @@ sharpen(void)
            i ++, trow = (trow + 1) & 3)
 	src_filters[i] = src_rows[trow];
 
+     /*
+      * Copy first and last pixels...
+      */
+
+      memcpy(dst_row, src_filters[1] + 0, img_bpp);
+      memcpy(dst_row + xmax, src_filters[1] + xmax, img_bpp);
+
+     /*
+      * Sharpen row...
+      */
+
       for (x = img_bpp, dst_ptr = dst_row + img_bpp;
            x < xmax;
            x ++, dst_ptr ++)
@@ -474,8 +468,14 @@ sharpen(void)
           *dst_ptr = i;
       };
 
-      gimp_pixel_rgn_set_row(&dst_rgn, dst_row + img_bpp, sel_x1 + 1, y, sel_width - 2);
-    };
+     /*
+      * Set the row...
+      */
+
+      gimp_pixel_rgn_set_row(&dst_rgn, dst_row, sel_x1, y, sel_width);
+    }
+    else if (count == 2)
+      gimp_pixel_rgn_set_row(&dst_rgn, src_rows[(row + 3) & 3], sel_x1, y, sel_width);
 
     if ((y & 15) == 0)
       gimp_progress_update((double)(y - sel_y1) / (double)sel_height);
@@ -1011,5 +1011,5 @@ dialog_close_callback(GtkWidget *widget,	/* I - Dialog window */
 
 
 /*
- * End of "$Id: sharpen.c,v 1.1 1998/02/20 10:41:24 yosh Exp $".
+ * End of "$Id: sharpen.c,v 1.2 1998/03/13 14:31:20 neo Exp $".
  */
